@@ -85,7 +85,7 @@ quota_t *quota_new (int q_type, int id, char *fs_spec)
   if (QF_IS_XFS(quota_format)) {
      output_debug("Detected quota format: XFS");
   }
-  else if (QF_IS_NEW(quota_format)) {
+  if (QF_IS_NEW(quota_format)) {
      output_debug("Detected quota format: NEW");
      if (IF_GENERIC) {
        output_debug("Detected quota interface: GENERIC");
@@ -105,7 +105,7 @@ quota_t *quota_new (int q_type, int id, char *fs_spec)
        output_debug("Detected quota interface: GENERIC");
      }
   }
-  else {
+  else if (! QF_IS_XFS(quota_format)) {
      output_error("Unknown quota format!");
      exit(ERR_SYS);
   }
@@ -472,8 +472,16 @@ int kern_quota_format(fs_t *fs, int q_type) {
    int ret = 0;
    struct stat st;
 
-   if (stat("/proc/fs/xfs/stat", &st) == 0)
-     quota_format |= (1 << QF_XFS);
+   if (strcasecmp(fs->mnt_type, "xfs") == 0) {
+      if (stat("/proc/fs/xfs/stat", &st) == 0) {
+	 quota_format |= (1 << QF_XFS);
+	 return ret;
+      }
+      else {
+	 output_error("%s is mounted as XFS but no kernel support for XFS quota!", fs->device);
+	 exit(ERR_SYS);
+      }
+   }
 
    if ((f = fopen("/proc/fs/quota", "r"))) {
       if (fscanf(f, "Version %u", &version) != 1) {
@@ -484,18 +492,23 @@ int kern_quota_format(fs_t *fs, int q_type) {
    }
    else if (stat("/proc/sys/fs/quota", &st) == 0) {
       /* Either QF_VFSOLD or QF_VFSV0 */
-      int actfmt;
+      int actfmt, retval;
       kernel_iface = IFACE_GENERIC;
-      if (quotactl(QCMD(Q_GETFMT, q_type), fs->device, 0, (void *) &actfmt) < 0) {
-	 output_error("Error while detecting kernel quota version: %s\n", strerror(errno));
-	 exit(ERR_SYS);
+      retval = quotactl(QCMD(Q_GETFMT, q_type), fs->device, 0, (void *) &actfmt);
+      if (retval < 0) {
+	 if (! QF_IS_XFS(quota_format)) {
+	    output_error("Error while detecting kernel quota version: %s\n", strerror(errno));
+	    exit(ERR_SYS);
+	 }
+      }
+      else {
+	 if (actfmt == 1)  /* Q_GETFMT retval for QF_VFSOLD */
+	    quota_format |= (1 << QF_VFSOLD);
+	 else if (actfmt == 2)  /* Q_GETFMT retval for QF_VFSV0 */
+	    quota_format |= (1 << QF_VFSV0);
+	 else
+	    return QF_ERROR;
       }
-      if (actfmt == 1)  /* Q_GETFMT retval for QF_VFSOLD */
-	 quota_format |= (1 << QF_VFSOLD);
-      else if (actfmt == 2)  /* Q_GETFMT retval for QF_VFSV0 */
-	 quota_format |= (1 << QF_VFSV0);
-      else
-	 return QF_ERROR;
       return ret;
    }
    else if (quotactl(QCMD(Q_V2_GETSTATS, 0), NULL, 0, (void *) &v2_stats) >= 0) {
